@@ -57,7 +57,7 @@ static bool owe_reset(struct owe_sm *owe)
 		return false;
 
 	owe->group = owe->ecc_groups[owe->retry];
-	owe->curve = l_ecc_curve_get_ike_group(owe->group);
+	owe->curve = l_ecc_curve_from_ike_group(owe->group);
 
 	if (owe->private)
 		l_ecc_scalar_free(owe->private);
@@ -193,8 +193,7 @@ static bool owe_compute_keys(struct owe_sm *owe, const void *public_key,
 		goto failed;
 
 	/* PMK = HKDF-expand(prk, "OWE Key Generation", n) */
-	if (!hkdf_expand(type, prk, nbytes, "OWE Key Generation",
-				strlen("OWE Key Generation"), pmk, nbytes))
+	if (!hkdf_expand(type, prk, nbytes, "OWE Key Generation", pmk, nbytes))
 		goto failed;
 
 	sha = l_checksum_new(type);
@@ -241,8 +240,8 @@ static int owe_rx_associate(struct auth_proto *ap, const uint8_t *frame,
 {
 	struct owe_sm *owe = l_container_of(ap, struct owe_sm, ap);
 
-	const struct mmpdu_header *mpdu = NULL;
-	const struct mmpdu_association_response *body;
+	const struct mmpdu_header *mpdu = (const struct mmpdu_header *) frame;
+	const struct mmpdu_association_response *body = mmpdu_body(mpdu);
 	struct ie_tlv_iter iter;
 	size_t owe_dh_len = 0;
 	const uint8_t *owe_dh = NULL;
@@ -250,14 +249,6 @@ static int owe_rx_associate(struct auth_proto *ap, const uint8_t *frame,
 	bool akm_found = false;
 	const void *data;
 
-	mpdu = mpdu_validate(frame, len);
-	if (!mpdu) {
-		l_error("could not process frame");
-		return -EBADMSG;
-	}
-
-	body = mmpdu_body(mpdu);
-
 	if (L_LE16_TO_CPU(body->status_code) ==
 				MMPDU_STATUS_CODE_UNSUPP_FINITE_CYCLIC_GROUP) {
 		if (!owe_retry(owe))
@@ -343,7 +334,7 @@ struct auth_proto *owe_sm_new(struct handshake_state *hs,
 	owe->auth_tx = auth;
 	owe->assoc_tx = assoc;
 	owe->user_data = user_data;
-	owe->ecc_groups = l_ecc_curve_get_supported_ike_groups();
+	owe->ecc_groups = l_ecc_supported_ike_groups();
 
 	owe->ap.start = owe_start;
 	owe->ap.free = owe_free;
